@@ -1,5 +1,6 @@
 import collections
 import itertools
+import math
 from functools import partial
 from typing import List, Optional, Tuple
 
@@ -673,6 +674,11 @@ class IrrepsData:
         irreps = Irreps(irreps)
         return IrrepsData(irreps, jnp.zeros(shape + (irreps.dim,)), [None] * len(irreps))
 
+    @staticmethod
+    def ones(irreps: Irreps, shape) -> "IrrepsData":
+        irreps = Irreps(irreps)
+        return IrrepsData(irreps, jnp.ones(shape + (irreps.dim,)), [jnp.ones(shape + (mul, ir.dim)) for mul, ir in irreps])
+
     @staticmethod
     def new(irreps: Irreps, any) -> "IrrepsData":
         r"""Create a new IrrepsData
@@ -772,6 +778,13 @@ class IrrepsData:
         ]
         return IrrepsData(self.irreps, self.contiguous, list)
 
+    def remove_nones(self) -> "IrrepsData":
+        if any(x is None for x in self.list):
+            irreps = [mul_ir for mul_ir, x in zip(self.irreps, self.list) if x is not None]
+            list = [x for x in self.list if x is not None]
+            return IrrepsData.from_list(irreps, list, self.shape)
+        return self
+
     def simplify(self) -> "IrrepsData":
         return self.convert(self.irreps.simplify())
 
@@ -783,6 +796,34 @@ class IrrepsData:
             for (i, j), contiguous in zip(zip([0] + indices, indices + [len(self.irreps)]), contiguous_parts)
         ]
 
+    def repeat_irreps_by_last_axis(self) -> "IrrepsData":
+        assert len(self.shape) >= 1
+        irreps = (self.shape[-1] * self.irreps).simplify()
+        contiguous = self.contiguous.reshape(self.shape[:-1] + (irreps.dim,))
+        return IrrepsData.from_contiguous(irreps, contiguous)
+
+    def repeat_mul_by_last_axis(self) -> "IrrepsData":
+        assert len(self.shape) >= 1
+        irreps = Irreps([(self.shape[-1] * mul, ir) for mul, ir in self.irreps])
+        list = [
+            None if x is None else x.reshape(self.shape[:-1] + (mul, ir.dim))
+            for (mul, ir), x in zip(irreps, self.list)
+        ]
+        return IrrepsData.from_list(irreps, list, self.shape[:-1])
+
+    def factor_irreps_to_last_axis(self) -> "IrrepsData":
+        raise NotImplementedError
+
+    def factor_mul_to_last_axis(self, factor=None) -> "IrrepsData":
+        if factor is None:
+            factor = math.gcd(*(mul for mul, _ in self.irreps))
+        irreps = Irreps([(mul // factor, ir) for mul, ir in self.irreps])
+        list = [
+            None if x is None else x.reshape(self.shape + (factor, mul, ir.dim))
+            for (mul, ir), x in zip(irreps, self.list)
+        ]
+        return IrrepsData.from_list(irreps, list, self.shape + (factor,))
+
     @partial(jax.jit, inline=True)
     def transform_by_angles(self, alpha, beta, gamma, k=0):
         r"""Rotate the data by angles according to the irreps
@@ -933,21 +974,48 @@ class IrrepsData:
         list = [x if y is None else (-y if x is None else x - y) for x, y in zip(self.list, other.list)]
         return IrrepsData(self.irreps, self.contiguous - other.contiguous, list)
 
+    def __mul__(self, other):
+        list = [None if x is None else x * other for x in self.list]
+        return IrrepsData(self.irreps, self.contiguous * other, list)
+
+    def __rmul__(self, other):
+        return self * other
+
+    def __truediv__(self, other):
+        list = [None if x is None else x / other for x in self.list]
+        return IrrepsData(self.irreps, self.contiguous / other, list)
+
     @staticmethod
-    def cat(args):
+    def cat(args, axis="irreps"):
         r"""Concatenate IrrepsData
 
         Args:
             args (list of `IrrepsData`): list of data to concatenate
+            axis (str or int): axis to concatenate on
 
         Returns:
             `IrrepsData`: concatenated data
         """
-        irreps = Irreps(sum([x.irreps for x in args], Irreps("")))
+        assert len(args) >= 1
+        if axis == "irreps":
+            irreps = Irreps(sum([x.irreps for x in args], Irreps("")))
+            return IrrepsData(
+                irreps,
+                jnp.concatenate([x.contiguous for x in args], axis=-1),
+                sum([x.list for x in args], [])
+            )
+        elif axis == "mul":
+            raise NotImplementedError
+
+        assert isinstance(axis, int)
+        assert {x.irreps for x in args} == {args[0].irreps}
+        while axis < 0:
+            axis += len(args[0].shape)
+        args = [x.replace_none_with_zeros() for x in args]  # TODO this could be optimized
         return IrrepsData(
-            irreps,
-            jnp.concatenate([x.contiguous for x in args], axis=-1),
-            sum([x.list for x in args], [])
+            args[0].irreps,
+            jnp.concatenate([x.contiguous for x in args], axis=axis),
+            [jnp.concatenate(xs, axis=axis) for xs in zip(*[x.list for x in args])]
         )
 
     @staticmethod
